#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pcpdir.h"
#include "pcplus.h"

char PCPDir [] = PCPDIR;

struct PcLocRec PcLoc;
struct PcAddrRec PcAddr;
struct PcSearchRec PcSearch;

/* get a line of input from the keyboard. Change the line feed to a NUL */
/* Use fgets rather than gets because gets is unsafe */
char * getLine(char *d, size_t len){
	char *p;
	fgets(d,len,stdin);
	p = strchr(d,'\n');
   if (p)
		*p = '\0';
   return d;
}

/* Strip trailing spaces from the string s of length c */
size_t Rtrim(char *s, size_t c){
	int i;
	for (i = c - 1; i >= 0; i--)
		if ( s[i] == ' ' ) s[i] = '\0';		/* strip trailing spaces */
		else break;
	return (i + 1);
}

/* We must null terminate the source string, but there may not be sufficient
	space to do so at the end of the source string, so we must copy to a
	temporary string first */
void Show(char *l, char *s,short int c){
	char *t = malloc (c + 1);              /* get temp */
	printf("%s",l);								/* show the legend */
	strncpy(t,s,c);								/* copy to temp */
	t[c] = '\0';									/* null terminate */
	Rtrim(t,c);										/* strip spaces */
	printf("%s\n",t); 		 					/* show result */
	free(t);											/* free temp */
}
#define SHOWPC(x,y) Show(x,PcLoc.y,sizeof(PcLoc.y))
#define SHOWADDR(x,y) Show(x,PcAddr.y,sizeof(PcAddr.y))

int main(int argc, char *argv[]){
	char input[256];
	long status;
	int i,dPts, recNo;
	char temp[10];
	sprintf(PcLoc.VLI,"%04d",(int)sizeof(struct PcLocRec));
	sprintf(PcAddr.VLI,"%04d",(int)sizeof(struct PcAddrRec));
	while (1){
enter:
		printf("\nEnter postcode (CR when done): ");
		getLine(input,256);
		if (input[0] == '\0')
			break;
		strcpy(PcLoc.Postcode,input);
restart:
		recNo = 1;
		do{
/*	Do the postcode lookup here.
	The postcode wanted is in PcLoc.Postcode.
	The return value, if positive, indicates how many matches there are for
	the given postcode. (Note that a given postcode may have more than one
	postcode record, which will typically have different streets.See GL54 5SP
	for an example.)
	If the return value is negative, an error has occurred.
*/
			status = GetPostcode(&PcLoc,recNo);
			if (status == -2){		/* not found - see if changed*/
				char oldPostcode[10],newPostcode[10];
				strncpy(oldPostcode,input,8);
				oldPostcode[8] = '\0';
				status = GetChange(oldPostcode,newPostcode);
				if (status > 0){
					strncpy(PcLoc.Postcode,newPostcode,8);
					status = GetPostcode(&PcLoc,recNo);
				}
			}
			if (status >= 0){
				printf("Found %d matches\r\n",(int)status);
        			SHOWPC("Postcode        : ",Postcode);
        			SHOWPC("Dep Street      : ",DepStreet);
        			SHOWPC("Street          : ",Street);
        			SHOWPC("Dep Locality    : ",DepLoc);
        			SHOWPC("Dbl Dep Loc     : ",DblDepLoc);
        			SHOWPC("Town            : ",Town);
					printf("County Source   : %c\r\n",PcLoc.CountySource);
        			SHOWPC("County          : ",County);
        			SHOWPC("County Abbr     : ",CountyAbbr);
        			SHOWPC("Mailsort Code   : ",Mailsort);
        			SHOWPC("STDCode         : ",STDCode);
        			SHOWPC("GridE           : ",GridE);
        			SHOWPC("GridN           : ",GridN);
        			SHOWPC("Ward Code       : ",WardCode9);
        			SHOWPC("Ward Name       : ",WardName);
        			SHOWPC("SHA Code        : ",SHACode);
        			SHOWPC("SHA Name        : ",NHSName);
        			SHOWPC("NHS Reg. Code   : ",NHSRegionCode);
        			SHOWPC("NHS Region      : ",NHSRegion);
        			SHOWPC("Delivery Pts    : ",DeliveryPoints);
					printf("Postcode Type   : %c\r\n",PcLoc.UserType);
        			SHOWPC("Constit. Code   : ",ConstituencyCode9);
        			SHOWPC("Constituency    : ",Constituency);
        			SHOWPC("Dev. Cons. Code : ",DCCCode);
        			SHOWPC("Devolved Cons.  : ",DCCName);
        			SHOWPC("TV Region       : ",TVRegion);
        			SHOWPC("Authority Code  : ",AuthorityCode9);
        			SHOWPC("Authority       : ",Laua);
        			SHOWPC("Trad. County    : ",TradCounty);
        			SHOWPC("Admin. County   : ",AdminCounty);
        			SHOWPC("Censation       : ",CensusCode);
        			SHOWPC("Cens. Label     : ",CensationLabel);
        			SHOWPC("Affluence       : ",Affluence);
        			SHOWPC("LifeStage       : ",LifeStage);
        			SHOWPC("Additional      : ",AdditionalCensusInfo);
					printf("Occupancy       : %c\r\n",PcLoc.OccupancyType);
					printf("Address Type    : %c\r\n",PcLoc.AddressType);
        			SHOWPC("PCT Code        : ",PCTCode9);
        			SHOWPC("PCT Name        : ",PCTName);
        			SHOWPC("EER Code        : ",EERCode9);
        			SHOWPC("EER Name        : ",EERName);
        			SHOWPC("Urban Rural     : ",UrbanRuralCode);
        			SHOWPC("Urb. Rur. Name  : ",UrbanRuralName);
        			SHOWPC("LEA Code        : ",LEACode);
        			SHOWPC("LEA Name        : ",LEAName);
        			SHOWPC("Latitute        : ",Latitude);
        			SHOWPC("Longitude       : ",Longitude);
        			SHOWPC("SOA Lower       : ",SOALower);
        			SHOWPC("SOA Middle      : ",SOAMiddle);
        			SHOWPC("Sub Country     : ",SubCountryName);
				strncpy(temp,PcLoc.DeliveryPoints,sizeof(PcLoc.DeliveryPoints));
				temp[sizeof(PcLoc.DeliveryPoints)] = '\0';
				dPts = atoi(temp);
				{
					char localities[2048] = "";
               char aliasLocality[36];
               long aliasLocalityRecs = GetAliasLocality(aliasLocality,1); /* get number of recs*/
               aliasLocality[35] = '\0';
               if (aliasLocalityRecs > 0){
						for (i = 1; i <= aliasLocalityRecs; i++){
                     GetAliasLocality(aliasLocality,i);
                     Rtrim(aliasLocality,35);
                     if (localities[0] != '\0') strcat(localities,",");
                     strcat(localities,aliasLocality);
                  }
                  printf("Alias localities: %s\n",localities);
               }
            }
				printf("\r\n");
				for (i = 1; i <= dPts; i++){
/* Read the address information here. The number of addresses for the given
	postcode record is in PcLoc.dPts  */
					GetAddress(&PcAddr,i);
					SHOWADDR("Org         : ",Organisation);
					SHOWADDR("Dep         : ",Department);
					SHOWADDR("Number      : ",Number);
					SHOWADDR("Building    : ",Building);
					SHOWADDR("Subbuilding : ",SubBuilding);
					SHOWADDR("PO Box      : ",POBox);
					SHOWADDR("Households  : ",Households);
					SHOWADDR("DPS         : ",Mailcode);
					SHOWADDR("UDPRN       : ",Udprn);
					SHOWADDR("Just Built  : ",JustBuilt);
					printf("\r\n");
					if (i != dPts || recNo != status){
						printf("R - Restart this postcode, N - Next postcode, E - Enter new postcode, CR to continue: ");
						fgets(input,256,stdin);
						switch(input[0]){
						case 'r' : case 'R' :
								goto restart;    /* start from beginning again */
						case 'n' : case 'N' :
								goto Break;			/* proceed to next postcode record */
						case 'e' : case 'E' :
								goto enter;	/* back to get more input */
						}
					}
				}
			} else {
				printf("Error %d\n", (int)status);
				break;
			}
Break:
	;
		} while (recNo++ < status);
	}
	printf("\n");
	exit(EXIT_SUCCESS);
}

