#ifndef pcplus_h
#define pcplus_h

#ifndef DLLCALL
#define DLLCALL
#endif

struct PcLocRec{
	char VLI[4];
	char Postcode[8];
	char DepStreet[60];
	char Street[60];
	char DblDepLoc[35];
	char DepLoc[35];
	char Town[30];
	char CountySource;		/* '0' if Royal Mail Data, '1' if Supplied by AFD */
	char County[30];
	char CountyAbbr[20];
	char Mailsort[5];
	char STDCode[8];
	char GridE[5];
	char GridN[5];
	char WardCode[6];
	char WardName[50];
	char NHSCode[3];
	char NHSName[50];
	char NHSRegionCode[3];
	char NHSRegion[40];
	char DeliveryPoints[3];
	char UserType;
	char Constituency[50];
	char TVRegion[30];
	char Laua[50];
	char TradCounty[30];
	char AdminCounty[30];
	char CensusCode[4];
	char Affluence[30];
	char LifeStage[100];
	char AdditionalCensusInfo[200];
	char OccupancyType;
	char AddressType;
	char PCTCode[3];
	char PCTName[50];
	char EERCode[2];
	char EERName[40];
	char UrbanRuralCode[2];
	char UrbanRuralName[60];
	char LEACode[3];
	char LEAName[50];

	char Latitude[7];
	char Longitude[7];

	char SOALower[9];
	char SOAMiddle[9];

	char LauaCode[4];
	char GridE6thDigit[1];
	char GridN6thDigit[1];

	char WardCode9[9];
	char PCTCode9[9];
	char ConstituencyCode9[9];
	char EERCode9[9];
	char AuthorityCode9[9];

	char SubCountryName[20];
	char SHACode[9];
	char DCCCode[9];
	char DCCName[50];
	char CensationLabel[50];

};

struct PcAddrRec{
	char VLI[4];
	char POBox[6];
	char Organisation[60];
	char Department[60];
	char Number[4];
	char SubBuilding[60];
	char Building[60];
	char Mailcode[2];
	char Households[4];
	char Udprn[8];
	char JustBuilt[10];
	char CensusCode[4];
	char CensusLabel[50];
	char Affluence[30];
	char Lifestage[100];
	char AdditionalCensusInfo[200];
};

struct PcLocBasicRec{
	char VLI[4];
	char Postcode[8];
	char DepStreet[60];
	char Street[60];
	char DblDepLoc[35];
	char DepLoc[35];
	char Town[30];
	char CountySource;		/* '0' if Royal Mail Data, '1' if Supplied by AFD */
	char County[30];
	char CountyAbbr[20];
	char DeliveryPoints[3];
	char UserType;
};

struct PcSearchRec{
	char VLI[4];
	char PcFrom[8];
	char PcTo[8];
	char STDCode[8];
	char Org[60];
	char Building[60];
	char DepStreet[60];
	char Street[60];
	char DblDepLoc[60];
	char DepLoc[60];
	char Town[60];
	char County[60];
	char JustBuilt[20];
	char GridE[6];
	char GridN[6];
	char Distance[6];
};

long DLLCALL GetPostcode(struct PcLocRec *PcLoc, long recNo);
typedef long (DLLCALL *GETPOSTCODE)(struct PcLocRec *, long recNo);

long DLLCALL GetAddress(struct PcAddrRec *ar, long recNo);
typedef long (DLLCALL *GETADDRESS)(struct PcAddrRec*, long recNo);

long DLLCALL InitiateSearch(struct PcSearchRec *ps, long SearchType);
typedef long (DLLCALL *INITIATESEARCH)(struct PcSearchRec *, long SearchType);

long DLLCALL FastFind(char *ps, long SearchType);
typedef long (DLLCALL *FASTFIND)(char *, long SearchType);

long DLLCALL GetMatch(struct PcLocRec *, long SkipToNextSector);
typedef long (DLLCALL *GETMATCH)(struct PcLocRec*, long SkipToNextSector);

long DLLCALL GetAliasLocality(char *dest, long recNo);
typedef long (DLLCALL *GETALIASLOCALITY)(char *dest, long recNo);

long DLLCALL GetAliasData(char *aliasData, long type, long recNo);
typedef long (DLLCALL *GETALIASDATA)(char *aliasData, long type, long recNo);

long DLLCALL GetChange(char * oldPostcode, char *newPostcode);
typedef long (DLLCALL *GETCHANGE)(char *oldPostcode, char *newPostcode);

/* the following are the values returned by GetChange */
#define FOUND 1
#define INVALID_POSTCODE -1
#define POSTCODE_NOT_FOUND -2
#define ERROR_OPENING_FILES -4

long DLLCALL LookupUdprn(struct PcLocRec * r, struct PcAddrRec *ar, char *udprn);
typedef long (DLLCALL *LOOKUPUDPRN)(struct PcLocRec * r, struct PcAddrRec *ar, char *udprn);

void DLLCALL CloseDown(void);
typedef void (DLLCALL *CLOSEDOWN)(void);



struct Pcplus {
	struct PcLocRec pcLoc;
	struct PcLocBasicRec pcLocBasic;
	struct PcAddrRec pcAddr;
	struct PcSearchRec pcSearch;
};

#define USE_WELSH_DATASET 0x10000000L
#define USE_ALTERNATE_GRIDREFS 0x01000000L

/* Returns */
#define INVALID_POSTCODE -1
#define POSTCODE_NOT_FOUND -2
#define INVALID_RECORD_NUMBER -3
#define ERROR_OPENING_FILES -4
#define FILE_READ_ERROR -5
#define END_OF_SEARCH -6
#define DATA_LICENCE_ERROR      -7

#endif

