#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "pcpdir.h"
#include "pcplus.h"

char PCPDir [] = PCPDIR;

struct PcLocRec PcLoc;
struct PcAddrRec PcAddr;

/* get a line of input from the keyboard. Change the line feed to a NUL */
/* Use fgets rather than gets because gets is unsafe */
char * getLine(char *d, size_t len){
	char *p;
	fgets(d,len,stdin);
	p = strchr(d,'\n');
   if (p)
		*p = '\0';
   return d;
}

/* Strip trailing spaces from the string s of length c */
size_t Rtrim(char *s, size_t c){
	int i;
	for (i = c - 1; i >= 0; i--)
		if ( s[i] == ' ' ) s[i] = '\0';		/* strip trailing spaces */
		else break;
	return (i + 1);
}

/* We must null terminate the source string, but there may not be sufficient
	space to do so at the end of the source string, so we must copy to a
	temporary string first */
void Show(char *l, char *s,short int c){
	char *t = malloc (c + 1);              /* get temp */
	printf("%s",l);								/* show the legend */
	strncpy(t,s,c);								/* copy to temp */
	t[c] = '\0';									/* null terminate */
	Rtrim(t,c);										/* strip spaces */
	printf("%s\n",t); 		 					/* show result */
	free(t);											/* free temp */
}
#define SHOWPC(x,y) Show(x,PcLoc.y,sizeof(PcLoc.y))
#define SHOWADDR(x,y) Show(x,PcAddr.y,sizeof(PcAddr.y))

int main(int argc, char *argv[]){
	char udprn[256];
	long status;
	int i,dPts, recNo;
	char temp[10];
	sprintf(PcLoc.VLI,"%04d",(int)sizeof(struct PcLocRec));
	sprintf(PcAddr.VLI,"%04d",(int)sizeof(struct PcAddrRec));
	while (1){
		printf("\nEnter UDPRN (CR when done): ");
		getLine(udprn,256);
		if (udprn[0] == '\0')
			break;
		/*	Do the UDPRN lookup here.
			The udprn wanted is in udprn.
			The return value, if positive, indicates success and Locality and Address records will be returned
			If the return value is negative, an error has occurred.
		*/
		status = LookupUdprn(&PcLoc,&PcAddr,udprn);
		if (status >= 0) {
			printf("Found Match:\r\n");
       			SHOWPC("Postcode        : ",Postcode);
       			SHOWPC("Dep Street      : ",DepStreet);
       			SHOWPC("Street          : ",Street);
       			SHOWPC("Dep Locality    : ",DepLoc);
       			SHOWPC("Dbl Dep Loc     : ",DblDepLoc);
       			SHOWPC("Town            : ",Town);
			    printf("County Source   : %c\r\n",PcLoc.CountySource);
       			SHOWPC("County          : ",County);
       			SHOWPC("County Abbr     : ",CountyAbbr);
       			SHOWPC("Mailsort Code   : ",Mailsort);
       			SHOWPC("STDCode         : ",STDCode);
       			SHOWPC("GridE           : ",GridE);
       			SHOWPC("GridN           : ",GridN);
       			SHOWPC("Ward Code       : ",WardCode9);
       			SHOWPC("Ward Name       : ",WardName);
       			SHOWPC("SHA Code        : ",SHACode);
       			SHOWPC("SHA Name        : ",NHSName);
       			SHOWPC("NHS Reg. Code   : ",NHSRegionCode);
       			SHOWPC("NHS Region      : ",NHSRegion);
       			SHOWPC("Delivery Pts    : ",DeliveryPoints);
			    printf("Postcode Type   : %c\r\n",PcLoc.UserType);
       			SHOWPC("Constit. Code   : ",ConstituencyCode9);
       			SHOWPC("Constituency    : ",Constituency);
       			SHOWPC("Dev. Cons. Code : ",DCCCode);
       			SHOWPC("Devolved Cons.  : ",DCCName);
       			SHOWPC("TV Region       : ",TVRegion);
       			SHOWPC("Authority Code  : ",AuthorityCode9);
       			SHOWPC("Authority       : ",Laua);
       			SHOWPC("Trad. County    : ",TradCounty);
       			SHOWPC("Admin. County   : ",AdminCounty);
       			SHOWPC("Censation       : ",CensusCode);
       			SHOWPC("Cens. Label     : ",CensationLabel);
       			SHOWPC("Affluence       : ",Affluence);
       			SHOWPC("LifeStage       : ",LifeStage);
       			SHOWPC("Additional      : ",AdditionalCensusInfo);
			    printf("Occupancy       : %c\r\n",PcLoc.OccupancyType);
			    printf("Address Type    : %c\r\n",PcLoc.AddressType);
       			SHOWPC("PCT Code        : ",PCTCode9);
       			SHOWPC("PCT Name        : ",PCTName);
       			SHOWPC("EER Code        : ",EERCode9);
       			SHOWPC("EER Name        : ",EERName);
       			SHOWPC("Urban Rural     : ",UrbanRuralCode);
       			SHOWPC("Urb. Rur. Name  : ",UrbanRuralName);
       			SHOWPC("LEA Code        : ",LEACode);
       			SHOWPC("LEA Name        : ",LEAName);
       			SHOWPC("Latitute        : ",Latitude);
       			SHOWPC("Longitude       : ",Longitude);
       			SHOWPC("SOA Lower       : ",SOALower);
       			SHOWPC("SOA Middle      : ",SOAMiddle);
       			SHOWPC("Sub Country     : ",SubCountryName);
				printf("\r\n");
				SHOWADDR("Org        : ",Organisation);
				SHOWADDR("Dep        : ",Department);
				SHOWADDR("Number     : ",Number);
				SHOWADDR("Building   : ",Building);
				SHOWADDR("Subbuilding: ",SubBuilding);
				SHOWADDR("PO Box     : ",POBox);
				SHOWADDR("Households : ",Households);
				SHOWADDR("DPS        : ",Mailcode);
				SHOWADDR("UDPRN      : ",Udprn);
				SHOWADDR("Just Built : ",JustBuilt);
			printf("\r\n");
		} else {
			printf("Error %d\n", (int)status);
		}
	}
	printf("\n");
	exit(EXIT_SUCCESS);
}

